Attribute VB_Name = "modSizeAndTrim"
Option Explicit
Option Compare Text
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' modSizeAndTrim
' By Chip Pearson, chip@cpearson.com    www.cpearson.com
' This module contains the TrimToNull, TrimToChar, and SizeString procedures.
' These procedures work in any application that supports VBA.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''
' This enum is used by SizeString
' to indicate whether the supplied text
' appears on the left or right side of
' result string.
''''''''''''''''''''''''''''''''''''''
Public Enum SizeStringSide
    TextLeft = 1
    TextRight = 2
End Enum

'''''''''''''''''''''''''''''''''''
' used by FormatMessage
'''''''''''''''''''''''''''''''''''
Private Const FORMAT_MESSAGE_ALLOCATE_BUFFER As Long = &H100
Private Const FORMAT_MESSAGE_ARGUMENT_ARRAY  As Long = &H2000
Private Const FORMAT_MESSAGE_FROM_HMODULE  As Long = &H800
Private Const FORMAT_MESSAGE_FROM_STRING  As Long = &H400
Private Const FORMAT_MESSAGE_FROM_SYSTEM  As Long = &H1000
Private Const FORMAT_MESSAGE_MAX_WIDTH_MASK  As Long = &HFF
Private Const FORMAT_MESSAGE_IGNORE_INSERTS  As Long = &H200
Private Const FORMAT_MESSAGE_TEXT_LEN  As Long = &HA0 ' from VC++ ERRORS.H file

'''''''''''''''''''''''''''''''''''
' Windows API Declare
'''''''''''''''''''''''''''''''''''
Private Declare Function PathCompactPathEx Lib "shlwapi.dll" Alias "PathCompactPathExA" ( _
    ByVal pszOut As String, _
    ByVal pszSrc As String, _
    ByVal cchMax As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function FormatMessage Lib "kernel32" _
    Alias "FormatMessageA" ( _
    ByVal dwFlags As Long, _
    ByVal lpSource As Any, _
    ByVal dwMessageId As Long, _
    ByVal dwLanguageId As Long, _
    ByVal lpBuffer As String, _
    ByVal nSize As Long, _
    ByRef Arguments As Long) As Long


Public Function ShortenTextToChars(InputText As String, _
            NumberOfCharacters As Long) As String
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' ShortenTextToChars
' By Chip Pearson, www.cpearson.com , chip@cpearson.com
' http://www.cpearon.com/excel/SizeString.htm#PathCompactEx
'
' This function returns a shortened version of the InputText parameter that is
' NumberOfCharacters in length. This function is primarily designed for use with
' fully-qualified file names. With a file name, the function will remove or truncate
' an element of the path (e.g., a folder name), replacing the removed text with the string
' "...". While this is intended for use with file names, it will work with any text string.
' When used on text that does not contain '\' characters, it typically just truncates the
' right side of InputText.
' Returns vbNullString is an error occurred.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    Dim ResString As String
    Dim Res As Long
    Dim ErrorNumber As Long
    Dim ErrorText As String
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' ensure that InputText is not an empty string
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If InputText = vbNullString Then
        MsgBox "The InputText parameter is an empty string"
        ShortenTextToChars = vbNullString
        Exit Function
    End If
        
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Test for NumberOfCharacters <= 3. If the InputText
    ' is 3 or fewer characters, PathCompactPathEx would replace the
    ' entire string with "...". We don't want that. Return the entire
    ' InputString.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If Len(InputText) <= 3 Then
        ShortenTextToChars = InputText
        Exit Function
    End If
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Test for NumberOfCharacters less than equal to  3.
    ' If the NumberOfCharacters <= 3, PathCompactPathEx would return
    ' "...". Instead, return the left-most characters and get out.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If NumberOfCharacters <= 3 Then
        ShortenTextToChars = Left(InputText, NumberOfCharacters)
        Exit Function
    End If
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Ensure we have a valid number of chacters. If NumberOfCharacters
    ' is less than or equal to 0, or greater than the length of
    ' the InputText, PathCompactPathEx will crash the application.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If NumberOfCharacters <= 0 Then
        MsgBox "The NumberOfCharacters must be greater than 0."
        ShortenTextToChars = vbNullString
        Exit Function
    End If

    If NumberOfCharacters > Len(InputText) Then
        '''''''''''''''''''''''''''''''''''''''''
        ' NumberOfCharacters is greater than the
        ' length of InputText. This would cause
        ' PathCompactPathEx to crash the applicaiton.
        ' Exit with an error.
        '''''''''''''''''''''''''''''''''''''''''
        MsgBox "The value of NumberOfCharactes is greater than the length of the InputText string."
        ShortenTextToChars = vbNullString
        Exit Function
    End If
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Here we test if the length of InputTex is equal to
    ' NumberOfCharacters. If they are equal, return the
    ' the entire string and exit. If we allowed
    ' PathCompactPathEx to process the string, it would truncate
    ' on the right and replace the last three characters on the
    ' right with "...". We don't want that behavior -- we
    ' want to return the entire string.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    If Len(InputText) = NumberOfCharacters Then
        ShortenTextToChars = InputText
        Exit Function
    End If
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' initialize the buffer. Use + 1 to provide space of
    ' the trailing NULL character placed at the end of
    ' the string by PathCompactPathEx.
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ResString = String$(NumberOfCharacters + 1, vbNullChar)
    
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' shorten the text with PathCompactPathEx
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Res = PathCompactPathEx(pszOut:=ResString, pszSrc:=InputText, _
             cchMax:=NumberOfCharacters, dwFlags:=0&)
    If Res = 0 Then
        ErrorNumber = Err.LastDllError
        ErrorText = GetSystemErrorMessageText(ErrorNumber)
        MsgBox "An error occurred with PathCompactPathEx" & vbCrLf & _
            "Error Number: " & CStr(ErrorNumber) & vbCrLf & _
            "Descrition:   " & ErrorText
        ShortenTextToChars = vbNullString
        Exit Function
    End If
    
    '''''''''''''''''''''''''''''''''''''''
    ' trim to get the charcters to the left
    ' of the vbNullChar character.
    '''''''''''''''''''''''''''''''''''''''
    ResString = TrimToNull(InputText:=ResString)
    
    '''''''''''''''''''''''''''''''''''''''
    ' return the result string
    '''''''''''''''''''''''''''''''''''''''
    ShortenTextToChars = ResString
            

End Function


Public Function TrimToNull(InputText As String) As String
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' TrimToNull
' By Chip Pearson, www.cpearson.com , chip@cpearson.com
' http://www.cpearon.com/excel/SizeString.htm#TrimToNull
'
' This function returns the portion of InputText that is to the left of the vbNullChar
' character (same as Chr(0)). Typically, this function is used with strings
' populated by Windows API procedures. It is generally not used for native
' VB Strings.
'
' If no vbNullChar is found, the entire Text string is returned.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim Pos As Integer
    
    Pos = InStr(1, InputText, vbNullChar)
    If Pos > 0 Then
        TrimToNull = Left(InputText, Pos - 1)
    Else
        TrimToNull = InputText
    End If

End Function

Public Function TrimToChar(InputText As String, TrimChar As String, _
    Optional SearchFromRight As Boolean = False) As String
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' TrimToChar
' By Chip Pearson, www.cpearson.com , chip@cpearson.com
' http://www.cpearon.com/excel/SizeString.htm#TrimToChar
'
' This function returns a portion of the string Text that is to the left of
' TrimChar. If SearchFromRight is omitted or False, the returned string
' is that string to the left of the FIRST occurrence of TrimChar. If
' SearchFromRight is True, the returned string is that string to the left of the
' LAST occurrance of TrimToChar. If TrimToChar is not found in the string,
' the entire Text string is returned. TrimChar may be more than one character.
' Comparison is done in Text mode (case does not matter).
' If TrimChar is an empty string, the entire Text string is returned.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim Pos As Integer
    
    ' Test to see if TrimChar is vbNullString. If so, return the whole string. If we
    ' don't test here and used InStr as in the next logic block, an empty string would
    ' be returned.
    If TrimChar = vbNullString Then
        TrimToChar = InputText
        Exit Function
    End If
    
    ' find the position in Text of TrimChar
    If SearchFromRight = True Then
        Pos = InStrRev(InputText, TrimChar, -1, vbTextCompare)
    Else
        Pos = InStr(1, InputText, TrimChar, vbTextCompare)
    End If
    ' return the sub string
    If Pos > 0 Then
        TrimToChar = Left(InputText, Pos - 1)
    Else
        TrimToChar = InputText
    End If

End Function




Public Function SizeString(Text As String, Length As Long, _
        Optional ByVal TextSide As SizeStringSide = TextLeft, Optional PadChar As String = " ") As String
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SizeString
' By Chip Pearson, www.cpearson.com , chip@cpearson.com
' http://www.cpearson.com/excel/SizeString.htm
'
' This procedure creates a string of a specified length. Text is the original string
' to include, and Length is the length of the result string. TextSide indicates whether
' Text should appear on the left (in which case the result is padded on the right with
' PadChar) or on the right (in which case the string is padded on the left). When padding on
' either the left or right, padding is done using the PadChar. character. If PadChar is omitted,
' a space is used. If PadChar is longer than one character, the left-most character of PadChar
' is used. If PadChar is an empty string, a space is used. If TextSide is neither
' TextLeft or TextRight, the procedure uses TextLeft.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim sPadChar As String

If Len(Text) >= Length Then
    ' if the source string is longer than the specified length, return the
    ' Length left characters
    SizeString = Left(Text, Length)
    Exit Function
End If

If Len(PadChar) = 0 Then
    ' PadChar is an empty string. use a space.
    sPadChar = " "
Else
    ' use only the first character of PadChar
    sPadChar = Left(PadChar, 1)
End If

If (TextSide <> TextLeft) And (TextSide <> TextRight) Then
    ' if TextSide was neither TextLeft nor TextRight, use TextLeft.
    TextSide = TextLeft
End If

If TextSide = TextLeft Then
    ' if the text goes on the left, fill out the right with spaces
    SizeString = Text & String(Length - Len(Text), sPadChar)
Else
    ' otherwise fill on the left and put the Text on the right
    SizeString = String(Length - Len(Text), sPadChar) & Text
End If

End Function


Private Function GetSystemErrorMessageText(ErrorNumber As Long) As String
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' GetSystemErrorMessageText
' http://www.cpearson.com/FormatMessage.htm
' By Chip Pearson, www.cpearson.com , chip@cpearson.com
'
' This function gets the system error message text that corresponds
' to the error code parameter ErrorCode. This value is the value returned
' by Err.LastDLLError or by GetLastError, or occasionally as the returned
' result of a Windows API function.
'
' These are NOT the error numbers returned by Err.Number (for these
' errors, use Err.Description to get the description of the error).
'
' In general, you should use Err.LastDllError rather than GetLastError
' because under some circumstances the value of GetLastError will be
' reset to 0 before the value is returned to VBA. Err.LastDllError will
' always reliably return the last error number raised in an API function.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim ErrorText As String
Dim TextLen As Long
Dim FormatMessageResult As Long
Dim LangID As Long

''''''''''''''''''''''''''''''''
' initialize the variables
''''''''''''''''''''''''''''''''
LangID = 0& 'default language
ErrorText = String$(FORMAT_MESSAGE_TEXT_LEN, vbNullChar)
TextLen = FORMAT_MESSAGE_TEXT_LEN

' Call FormatMessage to get the text of the error message text
' associated with ErrorNumber.
FormatMessageResult = FormatMessage( _
                        dwFlags:=FORMAT_MESSAGE_FROM_SYSTEM Or _
                                 FORMAT_MESSAGE_IGNORE_INSERTS, _
                        lpSource:=0&, _
                        dwMessageId:=ErrorNumber, _
                        dwLanguageId:=LangID, _
                        lpBuffer:=ErrorText, _
                        nSize:=TextLen, _
                        Arguments:=0&)
If FormatMessageResult = 0& Then
    ' An error occured. Display the error number, but
    ' don't call GetSystemErrorMessageText to get the
    ' text, which would likely cause the error again,
    ' getting us into a loop.

    MsgBox "An error occurred with the FormatMessage" & _
           " API functiopn call. Error: " & _
           CStr(Err.LastDllError) & _
           " Hex(" & Hex(Err.LastDllError) & ")."
    GetSystemErrorMessageText = vbNullString
    Exit Function
End If

' If FormatMessageResult is not zero, it is the number
' of characters placed in the ErrorText variable.
' Take the left FormatMessageResult characters and
' return that text.
ErrorText = Left$(ErrorText, FormatMessageResult)
GetSystemErrorMessageText = ErrorText

End Function


